@@ -1,9 +1,9 @@
 /*
- * $Id: main.h,v 1.109 2010/01/31 02:12:24 we7u Exp $
+ * $Id: main.h,v 1.112 2016/07/01 18:51:54 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2010  The Xastir Group
+ * Copyright (C) 2000-2016  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -210,6 +210,8 @@ typedef struct _selections {
     int weather_objects;
     int gauge_objects;
     int other_objects;
+    int aircraft_objects;
+    int vessel_objects;
 } Selections;
 extern Selections Select_;
 
@@ -313,6 +315,7 @@ extern char prox_min[30];
 extern char prox_max[30];
 extern time_t sec_old;
 extern time_t sec_clear;
+extern time_t aircraft_sec_clear;
 extern int trail_segment_time;
 extern int trail_segment_distance;
 extern int RINO_download_interval;
