@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: xa_config.c,v 1.187 2010/08/10 09:11:25 we7u Exp $
+ * $Id: xa_config.c,v 1.196 2016/07/01 18:51:54 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2010  The Xastir Group
+ * Copyright (C) 2000-2016  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -165,10 +165,7 @@ int get_string(char *option, char *value, int value_size) {
 
     if (fin == NULL) {
 
-        xastir_snprintf(config_file,
-            sizeof(config_file),
-            "%s",
-            get_user_base_dir(CONFIG_FILE));
+        get_user_base_dir(CONFIG_FILE, config_file, sizeof(config_file));
 
         // filecreate() refuses to create a new file if it already
         // exists.
@@ -356,9 +353,29 @@ float get_float(char *option, float low, float high, float def) {
 
 
 
-
-char *get_user_base_dir(char *dir) {
-    static char base[MAX_VALUE];
+// Note on get_user_base_dir
+//
+// The original implementation of this function used a static char array
+// to do all the path construction, and returned a pointer to that array.
+// 
+// This function was changed in Sept 2012 to remove its original use
+// of a single static char array.  That version was highly unsafe in a 
+// threaded environment, and was used routinely in multiple threads.
+// Even inserting mutexes to prevent simultaneous writes by multiple threads
+// wasn't good enough, because once the mutex was released there was no
+// guarantee that one thread would not write into the static array while
+// another thread was reading it.  The result was that it was sometimes 
+// possible for paths to get corrupted, especially if multiple logging
+// options and png snapshots were simultaneously selected.
+//
+// Prior to the change, one called this as:
+// ptr = get_user_base_dir("pathfragment");
+// and ptr was always a pointer to the static char array.  Now 
+// the caller is responsible for passing in the array into which we'll 
+// write.  This way, threads should never be clobbering each other's paths
+// returned from this routine.
+// 
+char *get_user_base_dir(char *dir, char * base, size_t base_size) {
     char *env_ptr;
 
    // fprintf(stderr,"base: %s \nxa_config_dir: %s\n", base, xa_config_dir);
@@ -366,35 +383,35 @@ char *get_user_base_dir(char *dir) {
     switch (xa_config_dir[0]) {
     case '/':
 	//have some path
-        xastir_snprintf(base, sizeof(base), "%s",xa_config_dir);
+        xastir_snprintf(base, base_size, "%s",xa_config_dir);
         break; 
 
     case '\0' : 
         // build from scratch
         xastir_snprintf(base,
-            sizeof(base),
+            base_size,
             "%s",
             ((env_ptr = getenv ("XASTIR_USER_BASE")) != NULL) ? env_ptr : user_dir);
 
         if (base[strlen (base) - 1] != '/')
-            strncat (base, "/", sizeof(base) - 1 - strlen(base));
+            strncat (base, "/", base_size - 1 - strlen(base));
 
-        strncat (base, ".xastir/", sizeof(base) - 1 - strlen(base));
+        strncat (base, ".xastir/", base_size - 1 - strlen(base));
         break ;
 
     default: 
         // Unqualified path
-        xastir_snprintf(base, sizeof(base), "%s",
+        xastir_snprintf(base, base_size, "%s",
             ((env_ptr = getenv ("PWD")) != NULL) ? env_ptr : user_dir);
 
        	if (base[strlen (base) - 1] != '/')
-            strncat (base, "/", sizeof(base) - 1 - strlen(base));
+            strncat (base, "/", base_size - 1 - strlen(base));
 
-        strncat (base, xa_config_dir, sizeof(base) - 1 - strlen(base));
+        strncat (base, xa_config_dir, base_size - 1 - strlen(base));
     }
     
     if (base[strlen (base) - 1] != '/')
-        strncat (base, "/", sizeof(base) - 1 - strlen(base));
+        strncat (base, "/", base_size - 1 - strlen(base));
 
     // Save base so we monkey around less later. 
     
@@ -402,7 +419,7 @@ char *get_user_base_dir(char *dir) {
 
     // Append dir and return 
     
-    return strncat(base, dir, sizeof(base) - 1 - strlen(base));
+    return strncat(base, dir, base_size - 1 - strlen(base));
 
 }
 
@@ -429,7 +446,8 @@ char *get_data_base_dir(char *dir) {
     if (base[strlen (base) - 1] != '/')
         strncat(base, "/", sizeof(base) - 1 - strlen(base));
 
-    return strncat(base, dir, sizeof(base) - 1 - strlen(base));
+    strncat(base, dir, sizeof(base) - 1 - strlen(base));
+    return base;
 }
 
 
@@ -476,10 +494,8 @@ void save_data(void)  {
 //        fprintf(stderr,"Store String Start\n");
 
     // The new file we'll create
-    xastir_snprintf(config_file_tmp,
-        sizeof(config_file_tmp),
-        "%s",
-        get_user_base_dir(CONFIG_FILE_TMP));
+    get_user_base_dir(CONFIG_FILE_TMP, config_file_tmp, 
+                      sizeof(config_file_tmp));
 
     // Save to the new config file
     fout = fopen (config_file_tmp, "a");
@@ -648,6 +664,8 @@ fprintf(stderr,"X:%d  y:%d\n", (int)x_return, (int)y_return);
         store_int (fout, "DISPLAY_STATION_WX_OBJ",        Select_.weather_objects);
         store_int (fout, "DISPLAY_WATER_GAGE_OBJ",        Select_.gauge_objects);
         store_int (fout, "DISPLAY_OTHER_OBJECTS",         Select_.other_objects);
+        store_int (fout, "DISPLAY_AIRCRAFT_OBJECTS",      Select_.aircraft_objects);
+        store_int (fout, "DISPLAY_VESSEL_OBJECTS",        Select_.vessel_objects);
 
         // display values
         store_int (fout, "DISPLAY_CALLSIGN",              Display_.callsign);
@@ -720,6 +738,14 @@ fprintf(stderr,"X:%d  y:%d\n", (int)x_return, (int)y_return);
             strncat (name, "RADIO_PORT", sizeof(name) - 1 - strlen(name));
             store_string (fout, name, devices[i].radio_port);
 
+            xastir_snprintf(name,
+                sizeof(name),
+                "%s",
+                name_temp);
+            strncat (name, "CONVERSE_CMD", sizeof(name) - 1 - strlen(name));
+            store_string (fout, name, devices[i].device_converse_string);
+
+
 #ifdef HAVE_DB
             
             xastir_snprintf(name,
@@ -940,6 +966,14 @@ fprintf(stderr,"X:%d  y:%d\n", (int)x_return, (int)y_return);
                 name_temp);
             strncat (name, "SETTIME", sizeof(name) - 1 - strlen(name));
             store_int (fout, name, devices[i].set_time);
+
+            xastir_snprintf(name,
+                sizeof(name),
+                "%s",
+                name_temp);
+            strncat (name, "TNCEXTRADELAY", sizeof(name) - 1 - strlen(name));
+            store_int (fout, name, devices[i].tnc_extra_delay);
+ 
         }
         /* TNC */
         store_int (fout, "TNC_LOG_DATA", log_tnc_data);
@@ -1052,6 +1086,8 @@ fprintf(stderr,"X:%d  y:%d\n", (int)x_return, (int)y_return);
         store_long (fout, "DEFAULT_STATION_OLD", (long)sec_old);
 
         store_long (fout, "DEFAULT_STATION_CLEAR", (long)sec_clear);
+        store_long (fout, "DEFAULT_AIRCRAFT_CLEAR", (long)aircraft_sec_clear);
+
         store_long(fout, "DEFAULT_STATION_REMOVE", (long)sec_remove);
         store_string (fout, "HELP_DATA", HELP_FILE);
 
@@ -1117,30 +1153,19 @@ fprintf(stderr,"X:%d  y:%d\n", (int)x_return, (int)y_return);
         (void)fclose (fout);
 
 
-        xastir_snprintf(config_file,
-            sizeof(config_file),
-            "%s",
-            get_user_base_dir(CONFIG_FILE));
+        get_user_base_dir(CONFIG_FILE, config_file, sizeof(config_file));
 
-        xastir_snprintf(config_file_bak1,
-            sizeof(config_file_bak1),
-            "%s",
-            get_user_base_dir(CONFIG_FILE_BAK1));
+        get_user_base_dir(CONFIG_FILE_BAK1, config_file_bak1, 
+                          sizeof(config_file_bak1));
 
-        xastir_snprintf(config_file_bak2,
-            sizeof(config_file_bak2),
-            "%s",
-            get_user_base_dir(CONFIG_FILE_BAK2));
+        get_user_base_dir(CONFIG_FILE_BAK2, config_file_bak2, 
+                          sizeof(config_file_bak2));
 
-        xastir_snprintf(config_file_bak3,
-            sizeof(config_file_bak3),
-            "%s",
-            get_user_base_dir(CONFIG_FILE_BAK3));
+        get_user_base_dir(CONFIG_FILE_BAK3, config_file_bak3, 
+                          sizeof(config_file_bak3));
  
-        xastir_snprintf(config_file_bak4,
-            sizeof(config_file_bak4),
-            "%s",
-            get_user_base_dir(CONFIG_FILE_BAK4));
+        get_user_base_dir(CONFIG_FILE_BAK4,config_file_bak4, 
+                          sizeof(config_file_bak4));
 
 
         //
@@ -1314,14 +1339,14 @@ void load_data_or_default(void) {
     char name_temp[20];
     char name[50];
     long temp;
-
+    char user_base_dir[MAX_VALUE];
 
     // Force the locale to a default so that we don't have
     // conversion problems due to LANG/LC_ALL/LC_CTYPE/LC_NUMERIC
     // environment variables.
     (void)setlocale(LC_NUMERIC, "C");
     (void)setlocale(LC_CTYPE, "C");
-
+    
     /* language */
     if (!get_string ("LANGUAGE", lang_to_use, sizeof(lang_to_use))
             || lang_to_use[0] == '\0') {
@@ -1391,7 +1416,10 @@ void load_data_or_default(void) {
 
     if (!get_string ("STATION_COMMENTS", my_comment, sizeof(my_comment))
             || my_comment[0] == '\0') {
-        xastir_snprintf (my_comment, sizeof(my_comment), "XASTIR-%s", XASTIR_SYSTEM);
+          // We used to put "XASTIR-Linux" (or similar) here...
+        //xastir_snprintf (my_comment, sizeof(my_comment), "XASTIR-%s", XASTIR_SYSTEM);
+        // Now we put the empty string.
+        xastir_snprintf (my_comment, sizeof(my_comment), "");
     }
     /* replacing defined MY_TRAIL_DIFF_COLOR from db.c, default 0 matches 
        default value of MY_TRAIL_DIFF_COLOR to show all mycall-ssids in 
@@ -1441,11 +1469,12 @@ void load_data_or_default(void) {
             "%s",
 #ifdef LPR_PATH
             // Path to LPR if defined
-            LPR_PATH);
+            LPR_PATH
 #else // LPR_PATH
             // Empty path
-            "");
+            ""
 #endif // LPR_PATH
+            );
     }
     if (!get_string ("PREVIEWER_PROGRAM", previewer_program, sizeof(previewer_program))
             || previewer_program[0] == '\0') {
@@ -1454,14 +1483,15 @@ void load_data_or_default(void) {
             "%s",
 #ifdef GV_PATH
             // Path to GV if defined
-            GV_PATH);
+            GV_PATH
 #else // GV_PATH
             // Empty path
-            "");
+            ""
 #endif // GV_PATH
+            );
     }
 
-    letter_style = get_int ("MAP_LETTERSTYLE", 0, 2, 1);
+    letter_style = get_int ("MAP_LETTERSTYLE", 0, 3, 0 );
 
     icon_outline_style = get_int ("MAP_ICONOUTLINESTYLE", 0, 3, 0);
 
@@ -1508,9 +1538,14 @@ void load_data_or_default(void) {
             "%s",
             "config/selected_maps.sys");
     }
+    
+    // get the base path for the user base directory, so we can use it over
+    // and over without having to call get_user_base_dir a godzillion times.
+    get_user_base_dir("",user_base_dir, sizeof(user_base_dir));
+
     // Check for old complete path, change to new short path if a
     // match
-    if (strncmp( get_user_base_dir(""), SELECTED_MAP_DATA, strlen(get_user_base_dir(""))) == 0)
+    if (strncmp( user_base_dir, SELECTED_MAP_DATA, strlen(user_base_dir)) == 0)
          xastir_snprintf(SELECTED_MAP_DATA,
             sizeof(SELECTED_MAP_DATA),
             "%s",
@@ -1525,7 +1560,7 @@ void load_data_or_default(void) {
     }
     // Check for old complete path, change to new short path if a
     // match
-    if (strncmp( get_user_base_dir(""), MAP_INDEX_DATA, strlen(get_user_base_dir(""))) == 0)
+    if (strncmp( user_base_dir, MAP_INDEX_DATA, strlen(user_base_dir)) == 0)
         xastir_snprintf(MAP_INDEX_DATA,
             sizeof(MAP_INDEX_DATA),
             "%s",
@@ -1556,7 +1591,7 @@ void load_data_or_default(void) {
     }
     // Check for old complete path, change to new short path if a
     // match
-    if (strncmp( get_user_base_dir(""), group_data_file, strlen(get_user_base_dir(""))) == 0)
+    if (strncmp( user_base_dir, group_data_file, strlen(user_base_dir)) == 0)
         xastir_snprintf(group_data_file,
              sizeof(group_data_file),
             "%s",
@@ -1723,6 +1758,8 @@ void load_data_or_default(void) {
     Select_.weather_objects = get_int ("DISPLAY_STATION_WX_OBJ", 0, 1, 1);
     Select_.gauge_objects = get_int ("DISPLAY_WATER_GAGE_OBJ", 0, 1, 1);
     Select_.other_objects = get_int ("DISPLAY_OTHER_OBJECTS", 0, 1, 1);
+    Select_.aircraft_objects = get_int ("DISPLAY_AIRCRAFT_OBJECTS", 0, 1, 1);
+    Select_.vessel_objects = get_int ("DISPLAY_VESSEL_OBJECTS", 0, 1, 1);
 
     // display values
     Display_.callsign = get_int ("DISPLAY_CALLSIGN", 0, 1, 1);
@@ -1798,6 +1835,18 @@ void load_data_or_default(void) {
                 sizeof(devices[i].radio_port),
                 "0");
 
+        xastir_snprintf(name,
+            sizeof(name),
+            "%s",
+            name_temp);
+        strncat (name, "CONVERSE_CMD", sizeof(name) - 1 - strlen(name));
+        if (!get_string (name, devices[i].device_converse_string, sizeof(devices[i].device_converse_string))
+                || (strlen(devices[i].device_converse_string) == 0))
+            xastir_snprintf(devices[i].device_converse_string,
+                sizeof(devices[i].device_converse_string),
+                "k");
+
+
 #ifdef HAVE_DB        
 
         xastir_snprintf(name,
@@ -2056,19 +2105,26 @@ void load_data_or_default(void) {
         strncat (name, "ONSTARTUP", sizeof(name) - 1 - strlen(name));
         devices[i].connect_on_startup = get_int (name, 0,1,0);
 
-                xastir_snprintf(name,
-                    sizeof(name),
-                    "%s",
-                    name_temp);
-                strncat (name, "GPSRETR", sizeof(name) - 1 - strlen(name));
-                devices[i].gps_retrieve = get_int (name, 0,255,DEFAULT_GPS_RETR);
+        xastir_snprintf(name,
+            sizeof(name),
+            "%s",
+            name_temp);
+        strncat (name, "GPSRETR", sizeof(name) - 1 - strlen(name));
+        devices[i].gps_retrieve = get_int (name, 0,255,DEFAULT_GPS_RETR);
 
-                xastir_snprintf(name,
-                    sizeof(name),
-                    "%s",
-                    name_temp);
-                strncat (name, "SETTIME", sizeof(name) - 1 - strlen(name));
-                devices[i].set_time = get_int (name, 0,1,0);
+        xastir_snprintf(name,
+            sizeof(name),
+            "%s",
+            name_temp);
+        strncat (name, "SETTIME", sizeof(name) - 1 - strlen(name));
+        devices[i].set_time = get_int (name, 0,1,0);
+
+        xastir_snprintf(name,
+            sizeof(name),
+            "%s",
+            name_temp);
+        strncat (name, "TNCEXTRADELAY", sizeof(name) - 1 - strlen(name));
+        devices[i].tnc_extra_delay = get_int (name, 0,9999999,0);
     }
 
     /* TNC */
@@ -2083,7 +2139,7 @@ void load_data_or_default(void) {
     }
     // Check for old complete path, change to new short path if a
     // match
-    if (strncmp( get_user_base_dir(""), LOGFILE_TNC, strlen(get_user_base_dir(""))) == 0)
+    if (strncmp( user_base_dir, LOGFILE_TNC, strlen(user_base_dir)) == 0)
          xastir_snprintf(LOGFILE_TNC,
             sizeof(LOGFILE_TNC),
             "%s",
@@ -2112,7 +2168,7 @@ void load_data_or_default(void) {
     }
     // Check for old complete path, change to new short path if a
     // match
-    if (strncmp( get_user_base_dir(""), LOGFILE_IGATE, strlen(get_user_base_dir(""))) == 0)
+    if (strncmp( user_base_dir, LOGFILE_IGATE, strlen(user_base_dir)) == 0)
          xastir_snprintf(LOGFILE_IGATE,
             sizeof(LOGFILE_IGATE),
             "%s",
@@ -2127,7 +2183,7 @@ void load_data_or_default(void) {
     }
     // Check for old complete path, change to new short path if a
     // match
-    if (strncmp( get_user_base_dir(""), LOGFILE_NET, strlen(get_user_base_dir(""))) == 0)
+    if (strncmp( user_base_dir, LOGFILE_NET, strlen(user_base_dir)) == 0)
         xastir_snprintf(LOGFILE_NET,
             sizeof(LOGFILE_NET),
             "%s",
@@ -2142,7 +2198,7 @@ void load_data_or_default(void) {
     }
     // Check for old complete path, change to new short path if a
     // match
-    if (strncmp( get_user_base_dir(""), LOGFILE_WX, strlen(get_user_base_dir(""))) == 0)
+    if (strncmp( user_base_dir, LOGFILE_WX, strlen(user_base_dir)) == 0)
         xastir_snprintf(LOGFILE_WX,
             sizeof(LOGFILE_WX),
             "%s",
@@ -2157,7 +2213,7 @@ void load_data_or_default(void) {
     }
     // Check for old complete path, change to new short path if a
     // match
-    if (strncmp( get_user_base_dir(""), LOGFILE_MESSAGE, strlen(get_user_base_dir(""))) == 0)
+    if (strncmp( user_base_dir, LOGFILE_MESSAGE, strlen(user_base_dir)) == 0)
         xastir_snprintf(LOGFILE_MESSAGE,
             sizeof(LOGFILE_MESSAGE),
             "%s",
@@ -2172,7 +2228,7 @@ void load_data_or_default(void) {
     }
     // Check for old complete path, change to new short path if a
     // match
-    if (strncmp( get_user_base_dir(""), LOGFILE_WX_ALERT, strlen(get_user_base_dir(""))) == 0)
+    if (strncmp( user_base_dir, LOGFILE_WX_ALERT, strlen(user_base_dir)) == 0)
         xastir_snprintf(LOGFILE_WX_ALERT,
             sizeof(LOGFILE_WX_ALERT),
             "%s",
@@ -2191,7 +2247,7 @@ void load_data_or_default(void) {
     gps_time = (time_t) get_long ("GPS_TIME", 1l, 86400l, 60l);
 
     /* POSIT RATE */
-    POSIT_rate = (time_t) get_long ("POSIT_RATE", 1l, 86400l, 30*60l);
+    POSIT_rate = (time_t) get_long ("POSIT_RATE", 0l, 86400l, 30*60l);
 
     /* OBJECT RATE */
     OBJECT_rate = (time_t) get_long ("OBJECT_RATE", 1l, 86400l, 30*60l);
@@ -2326,6 +2382,7 @@ void load_data_or_default(void) {
     sec_old = (time_t) get_long ("DEFAULT_STATION_OLD", 1l, 604800l, 4800l);
 
     sec_clear = (time_t) get_long ("DEFAULT_STATION_CLEAR", 1l, 604800l, 43200l);
+    aircraft_sec_clear = (time_t) get_long ("DEFAULT_AIRCRAFT_CLEAR", 1l, 604800l, 0l);
 
     sec_remove = get_long("DEFAULT_STATION_REMOVE", 1l, 604800*2, sec_clear*2);
 
