@@ -1,9 +1,9 @@
 /*
- * $Id: xa_config.h,v 1.12 2010/01/31 02:12:25 we7u Exp $
+ * $Id: xa_config.h,v 1.14 2012/11/01 18:57:19 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2010  The Xastir Group
+ * Copyright (C) 2000-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,7 +27,7 @@
 
 extern time_t next_time;
 
-char *get_user_base_dir(char *dir);
+char *get_user_base_dir(char *dir, char *dest, size_t dest_size);
 char *get_data_base_dir(char *dir);
 void save_data(void);
 void load_data_or_default(void);
